import json
import logging

import requests

LOGGER = logging.getLogger(__name__)


class RundeckAPI:
    def __init__(
        self,
        api_version,
        auth_token,
        base_url,
        max_executions_for_request=1000,
    ):
        self.api_version = api_version
        self.auth_token = auth_token
        self.base_url = base_url
        self.max_executions_for_request = max_executions_for_request
        self.url = f'{self.base_url}/{self.api_version}/'

        self.headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'X-Rundeck-Auth-Token': self.auth_token,
        }

    def _get(self, url_path, params=None):
        return requests.get(
            f'{self.url}{url_path}', headers=self.headers, params=params,
        )

    def _delete(self, url_path):
        return requests.delete(f'{self.url}{url_path}', headers=self.headers)

    def _post(self, url_path, data):
        return requests.post(
            f'{self.url}{url_path}', data=data, headers=self.headers,
        )

    def get_project_names(self):
        response = self._get(url_path='projects')
        if response.status_code == 200:
            return [project['name'] for project in response.json()]

        LOGGER.error('No project created')
        return []

    def get_project(self, name):
        response = self._get(url_path=f'project/{name}/')
        if response.status_code == 200:
            return response.json()

        LOGGER.error('%s project does not exist', name)
        return ''

    def get_job_ids_for_project(self, project_name):
        params = {'project': project_name}
        response = self._get(url_path='jobs', params=params)

        if response.status_code == 200:
            return [job['id'] for job in response.json()]

        LOGGER.error('No jobs for the %s project found', project_name)
        return []

    def get_executions_for_job(self, job_id):
        response = self._get(url_path=f'job/{job_id}/executions/')

        if response.status_code == 200:
            executions = response.json()['executions']
            return [execution['id'] for execution in executions]

        LOGGER.error('No executions for the %s job found', job_id)
        return []

    def get_executions_for_project(self, project_name, filters=None):
        if filters:
            params = filters

        response = self._get(
            url_path=f'project/{project_name}/executions/', params=params,
        )
        if response.status_code == 200:
            executions = response.json()['executions']
            return [execution['id'] for execution in executions]

        LOGGER.error(
            'No executions for the %s project with the parameters %s found',
            project_name,
            filters,
        )
        return []

    def delete_execution(self, execution_id):
        response = self._delete(url_path=f'execution/{execution_id}')

        return response.status_code

    def bulk_delete_executions(self, execution_ids):
        response = self._post(
            url_path='executions/delete/',
            data=json.dumps({'ids': execution_ids}),
        )

        return response.status_code



import logging

import click
import click_log

from rundeck_executions_cleanup.rundeck_api import RundeckAPI

__version__ = '0.1.3'

LOGGER = logging.getLogger(__name__)
click_log.basic_config(LOGGER)


@click.command()
@click.option(
    '--api_token', required=True, type=str, help='Rundeck API token.',
)
@click.option('--api_url', required=True, type=str, help='Rundeck API url.')
@click.option(
    '--api_version', required=True, type=int, help='Rundeck API version.',
)
@click.option(
    '--older_execution_days',
    required=True,
    type=str,
    help=(
        'Days that previous executions will be deleted. '
        'Example: 30d. Check olderFilter in Rundeck documentation for more '
        'details: '
        'https://docs.rundeck.com/docs/api/rundeck-api.html#execution-query'
    ),
)
@click.option(
    '--projects',
    default='all',
    type=str,
    help='Rundeck projects that will delete executions.',
)
@click.option(
    '--max_executions_for_request',
    default=1000,
    help='Maximum number of API requests',
    type=int,
)
@click_log.simple_verbosity_option(LOGGER)
def main(
    api_token,
    api_url,
    api_version,
    max_executions_for_request,
    older_execution_days,
    projects,
):
    rapi = RundeckAPI(
        base_url=api_url,
        api_version=api_version,
        auth_token=api_token,
        max_executions_for_request=max_executions_for_request,
    )
    LOGGER.info(f'Connect Rundeck API {api_url} version {api_version}.')

    if projects == 'all':
        LOGGER.warning(
            'No projects was specified, so cleaning will be in all Rundeck projects.'  # noqa: E501
        )
        projects = rapi.get_project_names()
    else:
        projects = projects.split(',')
        for project in projects:
            if not rapi.get_project(project):
                return
    LOGGER.info(
        f'Executions of {older_execution_days} ago will be deleted in the projects: {projects}.'  # noqa: E501
    )

    execution_ids = []
    for project in projects:
        executions = rapi.get_executions_for_project(
            project_name=project,
            filters={'olderFilter': older_execution_days},
        )
        execution_ids.extend(executions)

    rapi.bulk_delete_executions(execution_ids=execution_ids)
    LOGGER.info(f'{str(len(execution_ids))} executions have been deleted.')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rundeck_executions_cleanup']

package_data = \
{'': ['*']}

install_requires = \
['click-log>=0.3.2,<0.4.0', 'click>=7.1.1,<8.0.0', 'requests>=2.23.0,<3.0.0']

entry_points = \
{'console_scripts': ['rundeck-executions-cleanup = '
                     'rundeck_executions_cleanup:main']}

setup_kwargs = {
    'name': 'rundeck-executions-cleanup',
    'version': '0.1.3',
    'description': 'Python CLI to remove old Rundeck executions and log files',
    'long_description': '# Rundeck Executions Cleanup\n\nPython CLI to remove old Rundeck executions and log files\n\nThis project was based on the [RundeckLogfileCleanup](https://github.com/phwitservices/RundeckLogfileCleanup) repository.\n',
    'author': 'Lucas Souto',
    'author_email': 'lucassouto@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/instruct-br/rundeck-executions-cleanup',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
